﻿using System.Collections;
using System.Data.Common;

namespace Connected.ServiceModel.Client.Data;
internal sealed class DataParameterCollection : DbParameterCollection
{
	private readonly object _syncRoot = new();
	public DataParameterCollection()
	{
		Items = new();
	}

	private List<DbParameter> Items { get; }
	public override int Count => Items.Count;

	public override object SyncRoot => _syncRoot;

	public override int Add(object value)
	{
		if (value is not DbParameter parameter)
			throw new InvalidCastException(nameof(DbParameter));

		Items.Add(parameter);

		return Items.Count - 1;
	}

	public override void AddRange(Array values)
	{
		foreach (var item in values)
			Add(item);
	}

	public override void Clear()
	{
		Items.Clear();
	}

	public override bool Contains(object value)
	{
		return Items.Contains(value);
	}

	public override bool Contains(string value)
	{
		return Items.FirstOrDefault(f => string.Equals(f.ParameterName, value, StringComparison.OrdinalIgnoreCase)) is not null;
	}

	public override void CopyTo(Array array, int index)
	{
		foreach (var item in array)
		{
			Insert(index, item);
			index++;
		}
	}

	public override IEnumerator GetEnumerator()
	{
		return Items.GetEnumerator();
	}

	public override int IndexOf(object value)
	{
		if (value is not DbParameter parameter)
			throw new InvalidCastException(nameof(DbParameter));

		return Items.IndexOf(parameter);
	}

	public override int IndexOf(string parameterName)
	{
		if (Items.FirstOrDefault(f => string.Equals(f.ParameterName, parameterName, StringComparison.OrdinalIgnoreCase)) is DbParameter parameter)
			return Items.IndexOf(parameter);

		return -1;
	}

	public override void Insert(int index, object value)
	{
		if (value is not DbParameter parameter)
			throw new InvalidCastException(nameof(DbParameter));

		Items.Insert(index, parameter);
	}

	public override void Remove(object value)
	{
		if (value is not DbParameter parameter)
			throw new InvalidCastException(nameof(DbParameter));

		Items.Remove(parameter);
	}

	public override void RemoveAt(int index)
	{
		Items.RemoveAt(index);
	}

	public override void RemoveAt(string parameterName)
	{
		if (Items.FirstOrDefault(f => string.Equals(f.ParameterName, parameterName, StringComparison.OrdinalIgnoreCase)) is DbParameter parameter)
			Items.Remove(parameter);
	}

	protected override DbParameter GetParameter(int index)
	{
		return Items[index];
	}

	protected override DbParameter GetParameter(string parameterName)
	{
		return Items.First(f => string.Equals(f.ParameterName, parameterName, StringComparison.OrdinalIgnoreCase));
	}

	protected override void SetParameter(int index, DbParameter value)
	{
		Items[index] = value;
	}

	protected override void SetParameter(string parameterName, DbParameter value)
	{
		Items[IndexOf(parameterName)] = value;
	}
}
